@@ -61,4 +61,6 @@ SHEEPDOG = "SHEEPDOG"
 VMDK = "VMDK"
 GPFS = "GPFS"
 VERITAS_HYPERSCALE = "VERITAS_HYPERSCALE"
+STORPOOL = "STORPOOL"
 NVME = "NVME"
+NVMEOF = "NVMEOF"
