@@ -71,7 +71,7 @@ class RBDConnector(base.BaseLinuxConnector):
 
     def _check_or_get_keyring_contents(self, keyring, cluster_name, user):
         try:
-            if keyring is None:
+            if keyring is None and user:
                 keyring_path = ("/etc/ceph/%s.client.%s.keyring" %
                                 (cluster_name, user))
                 with open(keyring_path, 'r') as keyring_file:
