@@ -23,6 +23,7 @@ from os_brick.initiator import connector
 from os_brick.initiator.connectors import base
 from os_brick.initiator.connectors import fake
 from os_brick.initiator.connectors import iscsi
+from os_brick.initiator.connectors import nvme
 from os_brick.initiator import linuxfc
 from os_brick.privileged import rootwrap as priv_rootwrap
 from os_brick.tests import base as test_base
@@ -33,6 +34,8 @@ FAKE_SCSI_WWN = '1234567890'
 
 class ConnectorUtilsTestCase(test_base.TestCase):
 
+    @mock.patch.object(nvme.NVMeConnector, '_get_system_uuid',
+                       return_value=None)
     @mock.patch.object(iscsi.ISCSIConnector, 'get_initiator',
                        return_value='fakeinitiator')
     @mock.patch.object(linuxfc.LinuxFibreChannel, 'get_fc_wwpns',
@@ -46,6 +49,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
                                              multipath_result,
                                              mock_wwnns, mock_wwpns,
                                              mock_initiator,
+                                             mock_sysuuid,
                                              host='fakehost'):
         props_actual = connector.get_connector_properties('sudo',
                                                           MY_IP,
@@ -89,7 +93,7 @@ class ConnectorUtilsTestCase(test_base.TestCase):
     def test_brick_get_connector_properties(self):
         self._test_brick_get_connector_properties(False, False, False)
 
-    @mock.patch.object(priv_rootwrap, 'execute')
+    @mock.patch.object(priv_rootwrap, 'execute', return_value=('', ''))
     def test_brick_get_connector_properties_multipath(self, mock_execute):
         self._test_brick_get_connector_properties(True, True, True)
         mock_execute.assert_called_once_with('multipathd', 'show', 'status',
@@ -149,7 +153,7 @@ class ConnectorTestCase(test_base.TestCase):
 
     def test_get_connector_properties(self):
         with mock.patch.object(priv_rootwrap, 'execute') as mock_exec:
-            mock_exec.return_value = True
+            mock_exec.return_value = ('', '')
             multipath = True
             enforce_multipath = True
             props = base.BaseLinuxConnector.get_connector_properties(
